<?php
/**
 * @file
 * Provides Admin interface to  GCS.
 */

/**
 * Implements hook_admin().
 */
function gcs_admin($form, &$form_state)
{
    $errors = false;
    $openssl = extension_loaded('openssl');
    $directory = 'private://';
    $private_directory = file_prepare_directory($directory, 'FILE_EXIST');
    $info = libraries_load('google-api-php-client');

    if (!$openssl || !$private_directory || (isset($info['loaded']) && !$info['loaded']))
    {
        $errors = true;
        $form['gcs_warning_details'] = array(
            '#type' => 'fieldset',
            '#title' => t('Configuration Issues') ,
            '#collapsible' => true,
            '#collapsed' => false,
        );
        $documentation_link = l(t('documentation') , GCS_ONLINE_DOCUMENTATION_LINK);
        $form['gcs_warning_details']['general'] = array(
            '#type' => 'item',
            '#markup' => t('We have detected issues with the requirements of the
        modules installation. Please fix the issues below before continuing
        with the configuration of the module. Check the online !documentation
        and/or readme.txt located in the module folder.', array(
                '!documentation' => $documentation_link
            )) ,
        );
        $error_messages = array();
        if (isset($info['loaded']) && !$info['loaded'])
        {
            $error_messages[] = t('Unable to load the Google API PHP Library.
        Please download and install the library.');
        }
        if (!$openssl)
        {
            $error_messages[] = t('Unable to detect the OpenSSL PHP module.
        Ensure this module is installed on your server.');
        }
        if (!$private_directory)
        {
            $file_system_link = l(t('here') , 'admin/config/media/file-system');
            $error_messages[] = t('The private directory is not writable.
        Please configure this directory !here.', array(
                '!here' => $file_system_link
            ));
        }
        $list = theme('item_list', array(
            'items' => $error_messages,
            'title' => t('Issues Found') ,
            'type' => 'ul',
        ));
        $form['gcs_warning_details']['errors'] = array(
            '#type' => 'item',
            '#markup' => $list,
        );
    }
    $form['gcs_public_key_file'] = array(
        '#type' => 'managed_file',
        '#title' => t('Service Account Json File') ,
        '#default_value' => variable_get('gcs_public_key') ,
        '#description' => t('The public json file that all detail for
         your Google Cloud Storage account.') ,
        '#required' => true,
        '#upload_validators' => array(
            'file_validate_extensions' => array(
                'json'
            ) ,
        ) ,
        '#upload_location' => 'private://gcs/',
        '#process' => array(
            'gcs_admin_public_key_file_element_process'
        ) ,
    );
    $form['gcs_client_id'] = array(
        '#type' => 'textfield',
        '#title' => t('Client ID') ,
        '#default_value' => variable_get('gcs_client_id') ,
        '#required' => true,
        '#attributes' => array(
            'placeholder' => t('Enter service account Cient ID.') ,
        ) ,
    );
    $form['gcs_private_key_id'] = array(
        '#type' => 'textfield',
        '#title' => t('Service Account Private Key Id') ,
        '#default_value' => variable_get('gcs_private_key_id') ,
        '#required' => true,
        '#attributes' => array(
            'placeholder' => t('Enter Private key id from json file.') ,
        ) ,
    );

    $form['gcs_client_email'] = array(
        '#type' => 'textfield',
        '#title' => t('Client Email') ,
        '#default_value' => variable_get('gcs_client_email') ,
        '#required' => true,
        '#attributes' => array(
            'placeholder' => t('Enter Client Email from json file.') ,
        ) ,
    );

    $form['gcs_bucket_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Bucket Name') ,
        '#default_value' => variable_get('gcs_bucket_name') ,
        '#required' => true,
        '#attributes' => array(
            'placeholder' => t('Enter Your Bucket name') ,
        ) ,
    );

    $form['gcs_folder'] = array(
        '#type' => 'textfield',
        '#title' => t('Folder Name') ,
        '#default_value' => variable_get('gcs_folder') ,
        '#required' => true,
        '#attributes' => array(
            'placeholder' => t('Enter Folder name which use for Upload files folder wise') ,
        ) ,
    );

    if (variable_get('gcs_public_key', NULL) != NULL)
    {
        $form['test_connection'] = array(
            '#type' => 'submit',
            '#value' => t('@test', array(
                '@test' => GCS_ADMIN_TEST_CONNECTION_BUTTON
            )) ,
        );
    }

    $form['submit'] = array(
        '#type' => 'submit',
        '#value' => t('@save', array(
            '@save' => GCS_ADMIN_SUBMIT_BUTTON
        )) ,
        '#submit' => array(
            'gcs_admin_submit'
        ) ,
    );

    return $form;
}

/**
 * Hides the default upload button.
 *
 * @param array $element
 *   The remove element.
 * @param array $form_state
 *   The form state.
 * @param array $form
 *   The form array.
 *
 * @return mixed
 *   the form element.
 */
function gcs_admin_public_key_file_element_process(array $element, array & $form_state, array $form)
{
    $element = file_managed_file_process($element, $form_state, $form);
    $element['upload_button']['#access'] = false;
    if (isset($element['remove_button']['#submit']))
    {
        $element['remove_button']['#submit'][] = 'gcs_delete_key_file';
    }
    return $element;
}

/**
 * Validates the admin form.
 *
 * Tests the credentials the user entered and responds with a message.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function gcs_admin_validate(array $form, array & $form_state)
{
    if (isset($form_state['clicked_button']) && $form_state['clicked_button']['#value'] == GCS_ADMIN_TEST_CONNECTION_BUTTON)
    {
        $info = libraries_load('google-api-php-client');
        if ($info['loaded'])
        {
            $response = gcs_test_connection();
            if (empty($response))
            {
                form_set_error('gcs_public_key_file', t('Not authenticating. Upload true json file and also Bucket Name'));
            }
            if ($response['access_token'] && $response['bucket'])
            {
                drupal_set_message(t('Successfully  connect to your Google Cloud Storage App.'));
            }
        }
        else
        {
            form_set_error('form', t("The Google API PHP Client library was unable to be loaded."));
        }
    }
}

/**
 * Submit handler for admin form.
 *
 * Saves the variables and Json file.
 * in regards to signed urls.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function gcs_admin_submit(array $form, array & $form_state)
{
    if ($form_state['clicked_button']['#value'] == GCS_ADMIN_SUBMIT_BUTTON)
    {
        variable_set('gcs_client_id', $form_state['values']['gcs_client_id']);
        variable_set('gcs_bucket_name', $form_state['values']['gcs_bucket_name']);
        variable_set('gcs_folder', $form_state['values']['gcs_folder']);
        variable_set('gcs_private_key_id', $form_state['values']['gcs_private_key_id']);
        variable_set('gcs_client_email', $form_state['values']['gcs_client_email']);
        gcs_admin_save_key_file($form, $form_state);
    }
}

/**
 * Saves the key file.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function gcs_admin_save_key_file(array & $form, array & $form_state)
{
    // Load the file.
    $file = file_load($form_state['values']['gcs_public_key_file']);
    // Change status to permanent.
    $file->status = FILE_STATUS_PERMANENT;
    $file->replace = FILE_EXISTS_REPLACE;
    $uploaded = file_save($file);
    global $user;
    $account = $user;
    // Record that the module is using the file.
    file_usage_add($file, 'gcs', 'admin', $account->uid);
    if ($uploaded)
    {
        $real_path = drupal_realpath($file->uri);
        variable_set('gcs_public_key', $file->fid);
        variable_set('gcs_public_key_file_path', $real_path);
    }
}

/**
 * Remove the file. Called when the user deletes the p12 key.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function gcs_delete_key_file(array $form, array & $form_state)
{
    $json_fid = variable_get('gcs_public_key');
    if ($json_fid)
    {
        $file = file_load($json_fid);
        if ($file)
        {
            file_usage_delete($file, 'gcs');
            file_delete($file);
            variable_del('gcs_public_key');
            variable_del('gcs_public_key_file_path');
        }
        unset($file);
    }
}

